@@ -4,13 +4,14 @@
 An object-oriented framework for command-line apps.
 """
 
-__version__ = '0.5.0'
+__version__ = '0.6.0'
 
+from .app import App, FromParams
 from .model import init, load, reload
 from .params import param, Key
+from .toggle import Toggle as toggle, pick_toggled, toggle_param
 from .attrs import config_attr
 from .utils import lookup
 from .layers import *
 from .config import *
 from .errors import *
-
